import logging
import os
import json

from requests.exceptions import HTTPError
from brian_dashboard_manager.grafana.utils.request import Request, TokenRequest


logger = logging.getLogger(__name__)


def _datasource_exists(datasource_to_check, provisioned_datasources):
    """
    Checks if a datasource exists in the list of provisioned datasources

    A datasource exists iff all the config on the provisioned version
    is the same as the local datasource (identified by its name)

    :param datasource_to_check: datasource to check
    :param provisioned_datasources: list of provisioned datasources
    :return: True if datasource exists, False otherwise
    """

    if len(datasource_to_check.keys()) == 0:
        return True
    for datasource in provisioned_datasources:
        exists = all(datasource.get(key) == datasource_to_check.get(key)
                     for key in datasource_to_check)
        if exists:
            return True
    return False


def get_missing_datasource_definitions(request: Request, dir=None):
    """
    Returns a list of datasource definitions that are not yet provisioned

    :param request: Request session to use
    :param dir: directory to search for datasource definitions
    :return: generator of datasource definitions
    """
    datasource_dir = dir or os.path.join(
        os.path.dirname(__file__), '../datasources/')
    existing_datasources = get_datasources(request)

    for (dirpath, _, filenames) in os.walk(datasource_dir):
        for file in filenames:
            if not file.endswith('.json'):
                continue
            filename = os.path.join(dirpath, file)
            datasource = json.load(open(filename, 'r'))
            if not _datasource_exists(datasource, existing_datasources):
                yield datasource


def datasource_exists(request: TokenRequest, datasource):
    """
    Checks if a datasource exists in the organization
    the API token is registered to.

    A datasource exists iff all the config on the provisioned version
    is the same as the local datasource (identified by its name)


    :param request: TokenRequest object
    :param datasource: datasource to check
    :return: True if datasource exists, False otherwise
    """
    existing = get_datasources(request)
    exists = _datasource_exists(datasource, existing)
    name = datasource.get('name')
    duplicate_exists = any([ds['name'] == name for ds in existing])
    if not exists and duplicate_exists:
        delete_datasource(request, name)
        return False
    return exists


def get_datasources(request: Request):
    """
    Returns list of all datasources

    :param request: Request session to use
    :return: list of datasources
    """
    return request.get('api/datasources').json()


def create_datasource(request: TokenRequest, datasource: dict):
    """
    Creates a datasource for the organization
    the API token is registered to.

    :param request: TokenRequest object
    :param datasource: datasource to create
    :return: datasource definition
    """
    try:
        r = request.post('api/datasources', json=datasource)
        logger.info(f'Provisioned datasource: {datasource["name"]}')
    except HTTPError:
        logger.exception('Error when provisioning datasource')
        return None
    return r.json()


def delete_datasource(request: TokenRequest, name: str):
    """
    Deletes a datasource for the organization
    the API token is registered to.

    :param request: TokenRequest object
    :param name: name of datasource to delete
    """
    return request.delete(f'api/datasources/name/{name}').json()
